# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-08-19 10:24
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_simple_forum', '0010_forumcategory_parent'),
    ]

    operations = [
        migrations.CreateModel(
            name='Facebook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('facebook_id', models.CharField(max_length=100)),
                ('facebook_url', models.CharField(default='', max_length=200)),
                ('first_name', models.CharField(default='', max_length=200)),
                ('last_name', models.CharField(default='', max_length=200)),
                ('verified', models.CharField(default='', max_length=200)),
                ('name', models.CharField(default='', max_length=200)),
                ('language', models.CharField(default='', max_length=200)),
                ('hometown', models.CharField(default='', max_length=200)),
                ('email', models.CharField(db_index=True, default='', max_length=200)),
                ('gender', models.CharField(default='', max_length=200)),
                ('dob', models.DateField(blank=True, null=True)),
                ('location', models.CharField(default='', max_length=200)),
                ('timezone', models.CharField(default='', max_length=200)),
                ('accesstoken', models.CharField(default='', max_length=2000)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_facebook', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='GitHub',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('git_url', models.URLField()),
                ('git_id', models.CharField(max_length=50)),
                ('disk_usage', models.CharField(max_length=200)),
                ('private_gists', models.IntegerField(default=0)),
                ('public_gists', models.IntegerField(default=0)),
                ('public_repos', models.IntegerField(default=0)),
                ('hireable', models.BooleanField(default=False)),
                ('total_private_repos', models.IntegerField(default=0)),
                ('owned_private_repos', models.IntegerField(default=0)),
                ('following', models.IntegerField(default=0)),
                ('followers', models.IntegerField(default=0)),
                ('company', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=200)),
                ('user_from', models.DateTimeField()),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_github', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Google',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('google_id', models.CharField(default='', max_length=200)),
                ('google_url', models.CharField(default='', max_length=1000)),
                ('verified_email', models.CharField(default='', max_length=200)),
                ('family_name', models.CharField(default='', max_length=200)),
                ('name', models.CharField(default='', max_length=200)),
                ('picture', models.CharField(default='', max_length=200)),
                ('gender', models.CharField(default='', max_length=10)),
                ('dob', models.CharField(default='', max_length=50)),
                ('given_name', models.CharField(default='', max_length=200)),
                ('email', models.CharField(db_index=True, default='', max_length=200)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_google', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Twitter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('twitter_id', models.CharField(default='', max_length=100)),
                ('screen_name', models.CharField(default='', max_length=100)),
                ('oauth_token', models.CharField(default='', max_length=200)),
                ('oauth_secret', models.CharField(default='', max_length=200)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_twitter', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.contrib.auth.forms import AuthenticationForm
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
from django import forms
from .models import ForumCategory, Badge, Topic, Comment, UserProfile
from django.template.defaultfilters import slugify


class LoginForm(AuthenticationForm):

    def clean_username(self):
        email = self.cleaned_data['username']
        user = User.objects.filter(email=email)
        if not user:
            raise forms.ValidationError('Email is not registered.')
        elif not user[0].is_active:
            raise forms.ValidationError('Your account is not activated yet!')
        return email


class RegisterForm(forms.ModelForm):

    class Meta:
        model = User
        fields = ['email', 'first_name', 'username', 'password']


class CategoryForm(forms.ModelForm):
    class Meta:
        model = ForumCategory
        exclude = ('slug', 'created_by')

    def clean_title(self):
        if ForumCategory.objects.filter(slug=slugify(self.cleaned_data['title'])).exclude(id=self.instance.id):
            raise forms.ValidationError('Category with this Name already exists.')

        return self.cleaned_data['title']

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(CategoryForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super(CategoryForm, self).save(commit=False)
        instance.created_by = self.user
        instance.title = self.cleaned_data['title']
        if str(self.cleaned_data['is_votable']) == 'True':
            instance.is_votable = True
        else:
            instance.is_votable = False
        if str(self.cleaned_data['is_active']) == 'True':
            instance.is_active = True
        else:
            instance.is_active = False
        if not self.instance.id:
            instance.slug = slugify(self.cleaned_data['title'])

        if commit:
            instance.save()
        return instance


class BadgeForm(forms.ModelForm):
    class Meta:
        model = Badge
        exclude = ('slug',)

    def clean_title(self):
        if Badge.objects.filter(slug=slugify(self.cleaned_data['title'])).exclude(id=self.instance.id):
            raise forms.ValidationError('Badge with this Name already exists.')

        return self.cleaned_data['title']

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(BadgeForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super(BadgeForm, self).save(commit=False)
        instance.title = self.cleaned_data['title']
        if not self.instance.id:
            instance.slug = slugify(self.cleaned_data['title'])
        if commit:
            instance.save()
        return instance


class TopicForm(forms.ModelForm):

    class Meta:
        model = Topic
        exclude = ('slug', 'no_of_views', 'status', 'created_by', 'no_of_likes', 'no_of_votes', 'no_of_down_votes', 'tags')

    def clean_title(self):
        if Topic.objects.filter(slug=slugify(self.cleaned_data['title'])).exclude(id=self.instance.id):
            raise forms.ValidationError('Topic with this Name already exists.')

        return self.cleaned_data['title']

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(TopicForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super(TopicForm, self).save(commit=False)
        instance.title = self.cleaned_data['title']
        instance.description = self.cleaned_data['description']
        instance.category = self.cleaned_data['category']
        if not self.instance.id:
            instance.slug = slugify(self.cleaned_data['title'])
            instance.created_by = self.user
            instance.status = 'Draft'
        if commit:
            instance.save()
        return instance


class CommentForm(forms.ModelForm):

    class Meta:
        model = Comment
        fields = ('comment', 'topic')

    def clean_comment(self):
        if self.cleaned_data['comment']:
            return self.cleaned_data['comment']
        raise forms.ValidationError('This field is required')

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(CommentForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super(CommentForm, self).save(commit=False)
        instance.comment = self.cleaned_data['comment']
        instance.topic = self.cleaned_data['topic']
        print (self.cleaned_data)
        if not self.instance.id:
            instance.commented_by = self.user
            if 'parent' in self.cleaned_data.keys() and self.cleaned_data['parent']:
                instance.parent = self.cleaned_data['parent']
        if commit:
            instance.save()
        return instance


class UserProfileForm(forms.ModelForm):

    class Meta:
        model = UserProfile
        fields = ['badges']


class ChangePasswordForm(forms.Form):
    oldpassword = forms.CharField(max_length=50)
    newpassword = forms.CharField(max_length=50)
    retypepassword = forms.CharField(max_length=50)


class UserChangePasswordForm(forms.Form):
    newpassword = forms.CharField(max_length=50)
    retypepassword = forms.CharField(max_length=50)


class ForgotPasswordForm(forms.Form):
    email = forms.CharField(max_length=200)

    def clean_username(self):
        user = UserProfile.objects.filter(user__email=self.data.get("email"))
        if user:
            return self.data.get("email")
        else:
            raise forms.ValidationError(
                "User with this email ID doesn't exists.")



from django.conf.urls import url
from endless_pagination.views import AjaxListView
from .models import Topic
from .views import *

urlpatterns = [
    url(r'^$', TopicList.as_view(), name="topic_list"),
    url(r'^register/$', IndexView.as_view(), name="signup"),
    url(r'^forum/login/$', ForumLoginView.as_view(), name="user_login"),
    url(r'^fb_login/$', facebook_login, name="facebook_login"),
    url(r'^gp_login/$', google_login, name="google_login"),

    url(r'^topic/add/$', TopicAdd.as_view(), name="new_topic"),

    url(r'^topic/view/(?P<slug>[-\w]+)/$', TopicView.as_view(), name="view_topic"),
    url(r'^topic/like/(?P<slug>[-\w]+)/$', TopicLike.as_view(), name="like_topic"),
    url(r'^topic/follow/(?P<slug>[-\w]+)/$', TopicFollow.as_view(), name="follow_topic"),
    url(r'^topic/votes/(?P<slug>[-\w]+)/$', TopicVotes.as_view(), name="topic_votes"),
    url(r'^mentioned-users/(?P<topic_id>[-\w]+)/$', get_mentioned_user, name="get_mentioned_user"),
    url(r'^user/profile/(?P<user_name>[-\w]+)/$', ProfileView.as_view(), name="view_profile"),
    url(r'^change-password/$', UserChangePassword.as_view(), name="user_change_password"),
    url(r'^forgot-password/$', ForgotPasswordView.as_view(), name="forgot_password"),
    url(r'^comment/delete/(?P<comment_id>[-\w]+)/$', CommentDelete.as_view(), name="comment_delete"),
    url(r'^comment/edit/(?P<comment_id>[-\w]+)/$', CommentEdit.as_view(), name="comment_edit"),

    url(r'^categories/$', ForumCategoryList.as_view(), name="forum_categories"),
    url(r'^tags/$', ForumTagsList.as_view(), name="forum_tags"),
    url(r'^badges/$', ForumBadgeList.as_view(), name="forum_badges"),
    url(r'^profile/$', UserProfileView.as_view(), name="user_profile"),
    url(r'^upload/profile-pic/$', UserProfilePicView.as_view(), name="user_profile_pic"),
    url(r'^send-mail/settings/$', UserSettingsView.as_view(), name="user_settings"),

    url(r'^category/(?P<slug>[-\w]+)/$', ForumCategoryView.as_view(), name="forum_category_detail"),
    url(r'^tags/(?P<slug>[-\w]+)/$', ForumTagsView.as_view(), name="forum_tags_detail"),

    url(r'^user/(?P<user_name>[a-zA-Z0-9_-]+.*?)/$', UserDetailView.as_view(), name="user_details"),

    url(r'^comment/add/$', CommentAdd.as_view(), name="new_comment"),

    url(r'^dashboard/$', LoginView.as_view(), name="dashboard"),
    # url(r'^dashboard/$', DashboardView.as_view(), name="dashboard"),
    url(r'^logout/$', getout, name='out'),

    url(r'^dashboard/category/list/$', CategoryList.as_view(), name="categories"),
    url(r'^dashboard/category/add/$', CategoryAdd.as_view(), name="add_category"),
    url(r'^dashboard/category/delete/(?P<slug>[-\w]+)/$', CategoryDelete.as_view(), name="delete_category"),
    url(r'^dashboard/category/edit/(?P<slug>[-\w]+)/$', CategoryEdit.as_view(), name="edit_category"),
    url(r'^dashboard/category/view/(?P<slug>[-\w]+)/$', CategoryDetailView.as_view(), name="view_category"),

    url(r'^dashboard/badge/list/$', BadgeList.as_view(), name="badges"),
    url(r'^dashboard/badge/add/$', BadgeAdd.as_view(), name="add_badge"),
    url(r'^dashboard/badge/delete/(?P<slug>[-\w]+)/$', BadgeDelete.as_view(), name="delete_badge"),
    url(r'^dashboard/badge/edit/(?P<slug>[-\w]+)/$', BadgeEdit.as_view(), name="edit_badge"),
    url(r'^dashboard/badge/view/(?P<slug>[-\w]+)/$', BadgeDetailView.as_view(), name="view_badge"),

    url(r'^dashboard/users/list/$', UserList.as_view(), name="users"),
    url(r'^dashboard/users/delete/(?P<user_id>[a-zA-Z0-9_-]+.*?)/$', DashboardUserDelete.as_view(), name="delete_user"),
    url(r'^dashboard/users/status/(?P<user_id>[a-zA-Z0-9_-]+.*?)/$', UserStatus.as_view(), name="user_status"),
    url(r'^dashboard/users/view/(?P<user_id>[a-zA-Z0-9_-]+.*?)/$', UserDetail.as_view(), name="user_detail"),
    url(r'^dashboard/users/edit/(?P<user_id>[a-zA-Z0-9_-]+.*?)/$', DashboardUserEdit.as_view(), name="edit_user"),

    url(r'^dashboard/topics/list/$', DashboardTopicList.as_view(), name="topics"),
    url(r'^dashboard/topics/delete/(?P<slug>[-\w]+)/$', DashboardTopicDelete.as_view(), name="delete_topic"),
    url(r'^dashboard/topic/view/(?P<slug>[-\w]+)/$', TopicDetail.as_view(), name="topic_detail"),
    url(r'^dashboard/topic/status/(?P<slug>[-\w]+)/$', TopicStatus.as_view(), name="topic_status"),

    url(r'^dashboard/change-password/$', ChangePassword.as_view(), name="change_password"),

]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-05 14:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0002_auto_20160628_1211'),
    ]

    operations = [
        migrations.AddField(
            model_name='topic',
            name='tags',
            field=models.ManyToManyField(to='django_simple_forum.Tags'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-21 10:25
from __future__ import unicode_literals

from django.db import migrations, models
import django_simple_forum.models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0008_auto_20160707_1311'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='profile_pic',
            field=models.FileField(blank=True, max_length=500, null=True, upload_to=django_simple_forum.models.img_url),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-20 10:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Badge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, unique=True)),
                ('slug', models.CharField(max_length=50, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateTimeField(auto_now_add=True)),
                ('commented_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='commented_by', to=settings.AUTH_USER_MODEL)),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comment_parent', to='django_simple_forum.Comment')),
            ],
        ),
        migrations.CreateModel(
            name='ForumCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=1000)),
                ('is_active', models.BooleanField(default=False)),
                ('color', models.CharField(default='#999999', max_length=20)),
                ('is_votable', models.BooleanField(default=False)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Tags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, unique=True)),
                ('slug', models.CharField(max_length=50, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Timeline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('namespace', models.CharField(db_index=True, default='default', max_length=250)),
                ('event_type', models.CharField(db_index=True, max_length=250)),
                ('data', models.TextField(blank=True, null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('is_read', models.BooleanField(default=False)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='content_type_timelines', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created_on'],
            },
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=2000)),
                ('description', models.TextField()),
                ('status', models.CharField(choices=[('Draft', 'Draft'), ('Published', 'Published'), ('Disabled', 'Disabled')], max_length=10)),
                ('created_on', models.DateTimeField(auto_now=True)),
                ('updated_on', models.DateTimeField(auto_now=True)),
                ('no_of_views', models.IntegerField(default='0')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_simple_forum.ForumCategory')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('used_votes', models.IntegerField(default='0')),
                ('user_roles', models.CharField(choices=[('Admin', 'Admin'), ('Publisher', 'Publisher')], max_length=10)),
                ('badges', models.ManyToManyField(to='django_simple_forum.Badge')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserTopics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_followed', models.BooleanField(default=False)),
                ('followed_on', models.DateField(blank=True, null=True)),
                ('no_of_votes', models.IntegerField(default='0')),
                ('is_like', models.BooleanField(default=False)),
                ('topic', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_simple_forum.Topic')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='comment',
            name='topic',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='topic_comments', to='django_simple_forum.Topic'),
        ),
        migrations.AlterIndexTogether(
            name='timeline',
            index_together=set([('content_type', 'object_id', 'namespace')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-06 09:12
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_simple_forum.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_simple_forum', '0004_topic_no_of_likes'),
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('attached_file', models.FileField(blank=True, max_length=500, null=True, upload_to=django_simple_forum.models.img_url)),
                ('comment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_simple_forum.Comment')),
                ('uploaded_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments_user', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-08-29 12:13
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_simple_forum', '0011_facebook_github_google_twitter'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='mentioned',
            field=models.ManyToManyField(related_name='mentioned_users', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django import template
from django_simple_forum.models import ForumCategory, Tags, Badge, UserTopics, UserProfile
from django.db.models import Count
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User

register = template.Library()


@register.assignment_tag()
def get_categories():
    all_categories = ForumCategory.objects.annotate(num_topics=Count('topic')).order_by('-num_topics')[:10]
    return all_categories


@register.assignment_tag()
def get_tags():
    all_categories = Tags.objects.annotate(num_topics=Count('topic')).order_by('-num_topics')[:10]
    return all_categories


@register.assignment_tag()
def get_users():
    all_users = User.objects.annotate(num_topics=Count('topic')).order_by('-num_topics')[:10]
    return all_users


@register.assignment_tag()
def get_badges():
    all_badges = Badge.objects.filter()[:10]
    return all_badges


@register.filter
def is_topic_like(topic_id, user_id):
    user_topic = UserTopics.objects.filter(topic_id=topic_id, user_id=user_id).first()
    if user_topic:
        if user_topic.is_like:
            return True
    return False


@register.filter
def user_profile_pic(user_id):
    user = UserProfile.objects.filter(user_id=user_id).first()
    if user:
        return user.profile_pic
    return ''


@register.filter
def is_topic_followed(topic_id, user_id):
    user_topic = UserTopics.objects.filter(topic_id=topic_id, user_id=user_id).first()
    if user_topic:
        if user_topic.is_followed:
            return True
    return False


@register.filter
def sub_comments(sub_comment):
    l = []
    reply = sub_comment.comment_parent.all()
    if reply:
        l.append(sub_comment)
        for sc in reply:
            l.extend(sub_comments(sc))
        return l
    return [sub_comment]





import urllib
try:
    import urllib.request as urllib2
except ImportError:
    import urllib2
    import hashlib
import hmac
import base64
import socket
import requests

# Find a JSON parser
try:
    import simplejson as json
except ImportError:
    try:
        from django.utils import simplejson as json
    except ImportError:
        import json
_parse_json = json.loads

# Find a query string parser
try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs


class GraphAPI(object):
    def __init__(self, access_token=None, timeout=None):
        self.access_token = access_token
        self.timeout = timeout

    def get_object(self, id, **args):
        return self.request(id, args)

    def get_objects(self, ids, **args):
        args["ids"] = ",".join(ids)
        return self.request("", args)

    def get_connections(self, id, connection_name, **args):
        return self.request(id + "/" + connection_name, args)
    
    def put_object(self, parent_object, connection_name, **data):
        if self.access_token:
            return self.request(parent_object + "/" + connection_name,post_args=data)
        else:
            raise AssertionError("access token not provided")

    def delete_object(self, id):
        """Deletes the object with the given ID from the graph."""
        # x=self.request(id, post_args={"method": "delete"})
        params = urllib.parse.urlencode({"method": "delete", 'access_token': str(id)})
        u = requests.get("https://graph.facebook.com/" + str(id) + "?" + params)
        print (u)
        groups = u.json()
        print (groups)
        return groups

    def request(self, path, args=None, post_args=None):

        args = args or {}

        if self.access_token:
            if post_args is not None:
                post_args["access_token"] = self.access_token
            else:
                args["access_token"] = self.access_token
        post_data = None if post_args is None else urllib.urlencode(post_args)
        try:
            file = urllib2.urlopen("https://graph.facebook.com/" + path + "?" +
                                   urllib.parse.urlencode(args),
                                   post_data, timeout=self.timeout)
        except (urllib2.HTTPError, error):
            response = _parse_json(error.read())
            raise GraphAPIError(response)
        except TypeError:
            # Timeout support for Python <2.6
            if self.timeout:
                socket.setdefaulttimeout(self.timeout)
            file = urllib2.urlopen("https://graph.facebook.com/" + path + "?" +
                                   urllib.parse.urlencode(args), post_data)
        print (file)
        try:
            fileInfo = file.info()
            print (fileInfo)
            print (fileInfo.keys())
            response = ''
            # if 'maintype' in fileInfo.keys():
                # if fileInfo.maintype == 'text':
            response = _parse_json(file.read())
                # elif fileInfo.maintype == 'image':
                #     mimetype = fileInfo['content-type']
                #     response = {
                #         "data": file.read(),
                #         "mime-type": mimetype,
                #         "url": file.url,
                #     }
            # else:
            #     raise GraphAPIError('Maintype was not text or image')
        finally:
            file.close()
        if response and isinstance(response, dict) and response.get("error"):
            raise GraphAPIError(response["error"]["type"],
                                response["error"]["message"])
        return response

    def fql(self, query, args=None, post_args=None):
        """FQL query.

        Example query: "SELECT affiliations FROM user WHERE uid = me()"

        """
        args = args or {}
        if self.access_token:
            if post_args is not None:
                post_args["access_token"] = self.access_token
            else:
                args["access_token"] = self.access_token
        post_data = None if post_args is None else urllib.urlencode(post_args)

        """Check if query is a dict and
           use the multiquery method
           else use single query
        """
        if not isinstance(query, basestring):
            args["queries"] = query
            fql_method = 'fql.multiquery'
        else:
            args["query"] = query
            fql_method = 'fql.query'

        args["format"] = "json"

        try:
            file = urllib2.urlopen("https://api.facebook.com/method/" +
                                   fql_method + "?" + urllib.urlencode(args),
                                   post_data, timeout=self.timeout)
        except TypeError:
            # Timeout support for Python <2.6
            if self.timeout:
                socket.setdefaulttimeout(self.timeout)
            file = urllib2.urlopen("https://api.facebook.com/method/" +
                                   fql_method + "?" + urllib.urlencode(args),
                                   post_data)

        try:
            content = file.read()
            response = _parse_json(content)
            #Return a list if success, return a dictionary if failed
            if type(response) is dict and "error_code" in response:
                raise GraphAPIError(response)
        except (Exception, e):
            raise e
        finally:
            file.close()

        return response
    def extend_access_token(self, app_id, app_secret):
        """
        Extends the expiration time of a valid OAuth access token. See
        <https://developers.facebook.com/roadmap/offline-access-removal/
        #extend_token>

        """
        args = {
            "client_id": app_id,
            "client_secret": app_secret,
            "grant_type": "fb_exchange_token",
            "fb_exchange_token": self.access_token,
        }
        response = urllib2.urlopen("https://graph.facebook.com/oauth/"
                                   "access_token?" +
                                   urllib.parse.urlencode(args)).read().decode('utf-8')
        query_str = parse_qs(response)
        if "access_token" in query_str:
            result = {"accesstoken": query_str["access_token"][0]}
            if "expires" in query_str:
                result["expire"] = query_str["expires"][0]
            return result
        else:
            response = json.loads(response)
            raise GraphAPIError(response)


class GraphAPIError(Exception):
    def __init__(self, result):
        #Exception.__init__(self, message)
        #self.type = type
        self.result = result
        try:
            self.type = result["error_code"]
        except:
            self.type = ""

        # OAuth 2.0 Draft 10
        try:
            self.message = result["error_description"]
        except:
            # OAuth 2.0 Draft 00
            try:
                self.message = result["error"]["message"]
            except:
                # REST server style
                try:
                    self.message = result["error_msg"]
                except:
                    self.message = result

        Exception.__init__(self, self.message)


def get_user_from_cookie(cookies, app_id, app_secret):
    
    cookie = cookies.get("fbsr_" + app_id, "")
    if not cookie:
        return None
    parsed_request = parse_signed_request(cookie, app_secret)
    if not parsed_request:
        return None
    try:
        result = get_access_token_from_code(parsed_request["code"], "",app_id, app_secret)
    except GraphAPIError:
        return None
    result["uid"] = parsed_request["user_id"]
    return result


def parse_signed_request(signed_request, app_secret):

    try:
        l = signed_request.split('.', 2)
        encoded_sig = str(l[0])
        payload = str(l[1])
        sig = base64.urlsafe_b64decode(encoded_sig + "=" *
                                       ((4 - len(encoded_sig) % 4) % 4))
        data = base64.urlsafe_b64decode(payload + "=" *
                                        ((4 - len(payload) % 4) % 4))
    except IndexError:
        # Signed request was malformed.
        return False
    except TypeError:
        # Signed request had a corrupted payload.
        return False

    data = _parse_json(data)
    if data.get('algorithm', '').upper() != 'HMAC-SHA256':
        return False

    # HMAC can only handle ascii (byte) strings
    # http://bugs.python.org/issue5285
    app_secret = app_secret.encode('ascii')
    payload = payload.encode('ascii')

    expected_sig = hmac.new(app_secret,
                            msg=payload,
                            digestmod=hashlib.sha256).digest()
    if sig != expected_sig:
        return False

    return data

def get_app_access_token(app_id, app_secret):
    
    # Get an app access token
    args = {'grant_type': 'client_credentials',
            'client_id': app_id,
            'client_secret': app_secret}

    file = urllib2.urlopen("https://graph.facebook.com/oauth/access_token?" +
                           urllib.urlencode(args))

    try:
        result = file.read().split("=")[1]
    finally:
        file.close()

    return result


def get_access_token_from_code(code, redirect_uri, app_id, app_secret):

    args = {
        "code": code,
        "redirect_uri": redirect_uri,
        "client_id": app_id,
        "client_secret": app_secret,
    }
    # We would use GraphAPI.request() here, except for that the fact
    # that the response is a key-value pair, and not JSON.
    response = urllib2.urlopen("https://graph.facebook.com/oauth/access_token" +
                              "?" + urllib.parse.urlencode(args)).read().decode('utf-8')
    query_str = parse_qs(response)
    if "access_token" in query_str:
        result = {"access_token": query_str["access_token"][0]}
        if "expires" in query_str:
            result["expires"] = query_str["expires"][0]
        return result
    else:
        print (type(response))
        print (response)
        jsonResponse = json.loads(str(response))
        print (jsonResponse)
        # response = json.loads(response)
        encoding = response.info().get_content_charset('utf8')
        data = json.loads(response.read().decode(encoding))
        print (data)
        return data


def get_app_access_token(app_id, app_secret):
    
    # Get an app access token
    args = {'grant_type': 'client_credentials',
            'client_id': app_id,
            'client_secret': app_secret}

    file = urllib2.urlopen("https://graph.facebook.com/oauth/access_token?" +
                           urllib.parse.urlencode(args))

    try:
        result = file.read().decode('utf-8').split("=")[1]
    finally:
        file.close()

    return result


def auth_url(app_id, canvas_url, perms=None, **kwargs):
    url = "https://www.facebook.com/dialog/oauth?"
    kvps = {'client_id': app_id, 'redirect_uri': canvas_url}
    if perms:
        kvps['scope'] = ",".join(perms)
    kvps.update(kwargs)
    return url + urllib.parse.urlencode(kvps)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-08-18 13:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0009_userprofile_profile_pic'),
    ]

    operations = [
        migrations.AddField(
            model_name='forumcategory',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_simple_forum.ForumCategory'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-07 13:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0007_topic_no_of_votes'),
    ]

    operations = [
        migrations.AddField(
            model_name='topic',
            name='no_of_down_votes',
            field=models.IntegerField(default='0'),
        ),
        migrations.AddField(
            model_name='usertopics',
            name='no_of_down_votes',
            field=models.IntegerField(default='0'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-06 05:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0003_topic_tags'),
    ]

    operations = [
        migrations.AddField(
            model_name='topic',
            name='no_of_likes',
            field=models.IntegerField(default='0'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-07 06:27
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0005_attachment'),
    ]

    operations = [
        migrations.AddField(
            model_name='forumcategory',
            name='description',
            field=models.TextField(default=datetime.datetime(2016, 7, 7, 6, 27, 51, 32010)),
            preserve_default=False,
        ),
    ]



from django.http.response import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.contrib.auth import logout


class AdminMixin(object):

    def dispatch(self, request, *args, **kwargs):
        user = self.request.user
        if not user.is_superuser:
            return HttpResponseRedirect(reverse('django_simple_forum:topic_list'))
        return super(AdminMixin, self).dispatch(request, *args, **kwargs)


class UserMixin(object):

    def dispatch(self, request, *args, **kwargs):
        user = self.request.user
        if user:
            if user.is_active:
                return super(UserMixin, self).dispatch(request, *args, **kwargs)
            else:
                logout(self.request)
        return HttpResponseRedirect(reverse('django_simple_forum:topic_list'))



from django.contrib.auth import logout, login, load_backend
from django.http.response import HttpResponseRedirect, HttpResponse
from django.views.generic.edit import FormView
from django.core.urlresolvers import reverse
from .forms import LoginForm
from django.views.generic import (TemplateView, UpdateView,
                                  ListView, CreateView, DetailView, DeleteView, View)
from django.shortcuts import redirect, render, get_object_or_404
from .models import ForumCategory, Badge, Topic, STATUS, Tags, UserProfile, UserTopics, Timeline, Facebook, Google, Comment
from .mixins import AdminMixin, UserMixin
from django.http import JsonResponse
from .forms import CategoryForm, BadgeForm, RegisterForm, TopicForm, CommentForm, UserProfileForm, ChangePasswordForm, UserChangePasswordForm, ForgotPasswordForm
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
# from endless_pagination.views import AjaxListView
from microurl import google_mini
from django.conf import settings
import json
from datetime import datetime
from django.contrib.auth.hashers import check_password
from django.core.files import File
from urllib.parse import urlparse, parse_qsl
from .facebook import GraphAPI, get_access_token_from_code
import urllib
try:
    import urllib.request as urllib2
except ImportError:
    import urllib2
import requests
from .sending_mail import Memail
from django.template import Context
from django.contrib.sites.shortcuts import get_current_site
from django.template import loader
from django.utils.crypto import get_random_string
from django.db.models import Q


def timeline_activity(user, content_object, namespace, event_type):
    Timeline.objects.create(
        user=user,
        content_object=content_object,
        namespace=namespace,
        event_type=event_type,
    )


class LoginView(FormView):
    template_name = 'dashboard/dashboard_login.html'
    form_class = LoginForm

    def form_valid(self, form):
        user = form.get_user()
        if user.is_superuser:
            login(self.request, form.get_user())
            data = {'error': False, 'response': 'You have successfully logged into the dashboard'}
        else:
            data = {'error': True, 'response': 'You dont have access to login to dashboard'}
        return JsonResponse(data)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:dashboard'))

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            if request.user.is_superuser:
                template_name = 'dashboard/dashboard.html'
                return render(request, template_name)
            else:
                return HttpResponseRedirect(reverse('django_simple_forum:topic_list'))
        return super(LoginView, self).dispatch(request, *args, **kwargs)


class DashboardView(AdminMixin, TemplateView):
    template_name = 'dashboard/dashboard.html'


def getout(request):
    if not request.user.is_superuser:
        logout(request)
        return HttpResponseRedirect(reverse('django_simple_forum:topic_list'))
    else:
        logout(request)
        return HttpResponseRedirect(reverse('django_simple_forum:dashboard'))


class CategoryList(AdminMixin, ListView):
    model = ForumCategory
    template_name = 'dashboard/categories.html'
    context_object_name = 'categories_list'

    def get_context_data(self, **kwargs):
        context = super(CategoryList, self).get_context_data(**kwargs)
        categories_list = ForumCategory.objects.filter(parent=None)
        context['categories_list'] = categories_list
        return context

    def post(self, request, *args, **kwargs):
        categories_list = self.model.objects.all()

        if request.POST.get('is_active') == 'True':
            categories_list = categories_list.filter(status=True)
        if request.POST.get('search_text', ''):
            categories_list = categories_list.filter(
                title__icontains=request.POST.get('search_text')
            )
        return render(request, self.template_name, {'categories_list': categories_list})


class CategoryDetailView(AdminMixin, DetailView):
    model = ForumCategory
    template_name = 'dashboard/view_category.html'
    slug_field = "slug"
    context_object_name = 'category'

    def get_object(self):
        return get_object_or_404(ForumCategory, slug=self.kwargs['slug'])


class CategoryAdd(AdminMixin, CreateView):
    model = ForumCategory
    form_class = CategoryForm
    template_name = "dashboard/category_add.html"

    def form_valid(self, form):
        menu = form.save()
        if self.request.POST.get('parent'):
            menu.parent_id = self.request.POST.get('parent')
            menu.save()

        data = {'error': False, 'response': 'Successfully Created Category'}
        return JsonResponse(data)

    def get_form(self, form_class):
        return form_class(self.request.POST, user=self.request.user)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:categories'))

    def form_invalid(self, form):
        data = {'error': True, 'response': form.errors}
        return HttpResponse(json.dumps(data))

    def get_context_data(self, **kwargs):
        context = super(CategoryAdd, self).get_context_data(**kwargs)
        form = CategoryForm(self.request.GET)
        menus = ForumCategory.objects.filter(parent=None)
        context['form'] = form
        context['menus'] = menus
        return context


class CategoryDelete(AdminMixin, DeleteView):
    model = ForumCategory
    slug_field = 'slug'
    template_name = "dashboard/categories.html"
    success_url = '/forum/dashboard/categories/'

    def get_object(self):
        return get_object_or_404(ForumCategory, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:categories'))

    def post(self, request, *args, **kwargs):
        category = self.get_object()
        category.delete()
        return JsonResponse({'error': False, 'response': 'Successfully Deleted Category'})


class CategoryEdit(AdminMixin, UpdateView):
    model = ForumCategory
    template_name = "dashboard/category_add.html"
    form_class = CategoryForm
    context_object_name = 'category'

    def get_object(self):
        return get_object_or_404(ForumCategory, slug=self.kwargs['slug'])

    def form_valid(self, form):
        menu = form.save()
        if self.request.POST.get('parent'):
            menu.parent_id = self.request.POST.get('parent')
            menu.save()
        data = {'error': False, 'response': 'Successfully Edited Category'}
        return JsonResponse(data)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:categories'))

    def get_form(self, form_class):
        return form_class(self.request.POST, instance=self.get_object(), user=self.request.user)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(CategoryEdit, self).get_context_data(**kwargs)
        form = CategoryForm(self.request.GET)
        menus = ForumCategory.objects.filter(parent=None)
        context['form'] = form
        context['menus'] = menus
        return context


class BadgeList(AdminMixin, ListView):
    model = Badge
    template_name = 'dashboard/badges.html'
    context_object_name = 'badges_list'

    def get_context_data(self, **kwargs):
        context = super(BadgeList, self).get_context_data(**kwargs)
        return context

    def post(self, request, *args, **kwargs):
        badges_list = self.model.objects.all()
        if request.POST.get('search_text', ''):
            badges_list = badges_list.filter(Q(title__icontains=request.POST.get('search_text')))
        return render(request, self.template_name, {'badges_list': badges_list, "per_page": request.POST.get("filter_per_page") if request.POST.get("filter_per_page") else 10})


class DashboardTopicList(AdminMixin, ListView):
    model = Topic
    template_name = 'dashboard/topics.html'
    context_object_name = 'topic_list'
    queryset = Topic.objects.filter().order_by('-created_on')

    def get_context_data(self, **kwargs):
        context = super(DashboardTopicList, self).get_context_data(**kwargs)
        return context

    def post(self, request, *args, **kwargs):
        topic_list = self.model.objects.all()
        if request.POST.get('search_text', ''):
            topic_list = topic_list.filter(Q(title__icontains=request.POST.get('search_text')) | Q(created_by__username__icontains=request.POST.get('search_text')))
        return render(request, self.template_name, {'topic_list': topic_list, "per_page": request.POST.get("filter_per_page") if request.POST.get("filter_per_page") else 10})


class BadgeDetailView(AdminMixin, DetailView):
    model = Badge
    template_name = 'dashboard/view_badge.html'
    slug_field = "slug"
    context_object_name = 'badge'

    def get_object(self):
        return get_object_or_404(Badge, slug=self.kwargs['slug'])


class BadgeAdd(AdminMixin, CreateView):
    model = Badge
    form_class = BadgeForm
    template_name = "dashboard/badge_add.html"

    def form_valid(self, form):
        form.save()
        data = {'error': False, 'response': 'Successfully Created Badge'}
        return JsonResponse(data)

    def get_form(self, form_class):
        return form_class(self.request.POST, user=self.request.user)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:badges'))

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(BadgeAdd, self).get_context_data(**kwargs)
        form = BadgeForm(self.request.GET)
        context['form'] = form
        return context


class BadgeDelete(AdminMixin, DeleteView):
    model = Badge
    slug_field = 'slug'
    template_name = "dashboard/badges.html"
    success_url = '/forum/dashboard/badges/'

    def get_object(self):
        return get_object_or_404(Badge, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:badges'))

    def post(self, request, *args, **kwargs):
        badge = self.get_object()
        badge.delete()
        return JsonResponse({'error': False, 'response': 'Successfully Deleted Badge'})


class BadgeEdit(AdminMixin, UpdateView):
    model = Badge
    template_name = "dashboard/badge_add.html"
    form_class = BadgeForm
    context_object_name = 'badge'

    def get_object(self):
        return get_object_or_404(Badge, slug=self.kwargs['slug'])

    def form_valid(self, form):
        form.save()
        data = {'error': False, 'response': 'Successfully Edited Badge'}
        return JsonResponse(data)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:badges'))

    def get_form(self, form_class):
        return form_class(self.request.POST, instance=self.get_object(), user=self.request.user)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(BadgeEdit, self).get_context_data(**kwargs)
        form = BadgeForm(self.request.GET)
        context['form'] = form
        return context


class UserList(AdminMixin, ListView):
    model = UserProfile
    template_name = 'dashboard/users.html'
    context_object_name = 'users_list'
    queryset = UserProfile.objects.filter()

    def get_context_data(self, **kwargs):
        context = super(UserList, self).get_context_data(**kwargs)
        return context

    def post(self, request, *args, **kwargs):
        users_list = self.model.objects.all()
        if request.POST.get('search_text', ''):
            users_list = list(set(users_list.filter(
                user__email__icontains=request.POST.get('search_text')
            ) | users_list.filter(
                user__username__icontains=request.POST.get('search_text')
            )))
        return render(request, self.template_name, {'users_list': users_list, "per_page": request.POST.get("filter_per_page") if request.POST.get("filter_per_page") else 10})


class DashboardUserEdit(AdminMixin, UpdateView):
    model = UserProfile
    template_name = "dashboard/edit_user.html"
    form_class = UserProfileForm
    context_object_name = 'user_profile'

    def get_object(self):
        return get_object_or_404(UserProfile, user_id=self.kwargs['user_id'])

    def form_valid(self, form):
        print (form.cleaned_data)
        user_profile = form.save()
        user_profile.badges.clear()
        user_profile.badges.add(*form.cleaned_data['badges'])
        data = {'error': False, 'response': 'Successfully Edited User'}
        return JsonResponse(data)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:users'))

    def get_form(self, form_class):
        return form_class(self.request.POST, instance=self.get_object())

    def form_invalid(self, form):
        print (self.request.POST)
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(DashboardUserEdit, self).get_context_data(**kwargs)
        form = UserProfileForm(self.request.GET)
        badges = Badge.objects.filter()
        context['form'] = form
        context['badges'] = badges
        context['user_profile'] = self.get_object()
        return context


class IndexView(FormView):
    template_name = 'forum/topic_list.html'
    form_class = RegisterForm

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        topics = Topic.objects.filter(status='Published')
        context['topics'] = topics
        return context

    def form_valid(self, form):
        user = User.objects.create(
            username=form.cleaned_data['username'], email=form.cleaned_data['email'])
        user.set_password(form.cleaned_data['password'])
        user.first_name = form.cleaned_data['first_name']
        user.is_active = True
        user.save()
        user.backend = 'django.contrib.auth.backends.ModelBackend'
        UserProfile.objects.create(user=user, user_roles='Publisher')
        login(self.request, user)

        timeline_activity(user=user, content_object=user,
                          namespace='created on', event_type="user-create")

        data = {'error': False, 'response': 'Successfully Created Badge'}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class ForumLoginView(FormView):
    template_name = 'forum/topic_list.html'
    form_class = LoginForm

    def get_context_data(self, **kwargs):
        context = super(ForumLoginView, self).get_context_data(**kwargs)
        topics = Topic.objects.filter(status='Published')
        context['topics'] = topics
        return context

    def form_valid(self, form):
        login(self.request, form.get_user())
        data = {'error': False, 'response': 'Successfully user loggedin'}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class TopicAdd(UserMixin, CreateView):
    model = Topic
    form_class = TopicForm
    template_name = "forum/new_topic.html"

    def form_valid(self, form):
        topic = form.save()
        if self.request.POST['sub_category']:
            topic.category_id = self.request.POST['sub_category']
        topic.save()
        if 'tags' in form.cleaned_data.keys() and form.cleaned_data['tags']:
            for tag in form.cleaned_data['tags'].split(','):
                if not Tags.objects.filter(slug=slugify(tag)):
                    each = Tags.objects.create(slug=slugify(tag), title=tag)
                    topic.tags.add(each)
                else:
                    each = Tags.objects.filter(slug=slugify(tag)).first()
                    topic.tags.add(each)
        liked_users_ids = UserTopics.objects.filter(topic__category=topic.category, is_like=True).values_list('user', flat=True)
        followed_users = UserTopics.objects.filter(topic__category=topic.category, is_followed=True).values_list('user', flat=True)
        # users = UserProfile.objects.filter(user_id__in=set(all_users))

        # for user in users:
        #     mto = [user.user.email]
        #     c = Context({'comment': comment, "user": user.user, 'topic_url': settings.HOST_URL + reverse('django_simple_forum:view_topic', kwargs={'slug': topic.slug}), "HOST_URL": settings.HOST_URL})
        #     print (c)
        #     t = loader.get_template('emails/new_topic.html')
        #     subject = "New Topic For The Category" + (topic.category.title)
        #     rendered = t.render(c)
        #     mfrom = settings.DEFAULT_FROM_EMAIL
        #     Memail(mto, mfrom, subject, rendered)

        timeline_activity(user=self.request.user, content_object=self.request.user,
                          namespace='created topic on', event_type="topic-create")

        data = {'error': False, 'response': 'Successfully Created Topic'}
        return JsonResponse(data)

    def get_form(self, form_class):
        return form_class(self.request.POST, user=self.request.user)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:signup'))

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(TopicAdd, self).get_context_data(**kwargs)
        form = TopicForm(self.request.GET)
        context['form'] = form
        context['status'] = STATUS
        context['categories'] = ForumCategory.objects.filter(
            is_active=True, is_votable=True, parent=None)
        context['sub_categories'] = ForumCategory.objects.filter(
            is_active=True, is_votable=True).exclude(parent=None)
        return context


class TopicList(ListView):
    queryset = Topic.objects.filter(status='Published').order_by('-created_on')
    template_name = 'forum/topic_list.html'
    context_object_name = "topic_list"

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            if request.user.is_superuser:
                return HttpResponseRedirect(reverse('django_simple_forum:dashboard'))
        return super(TopicList, self).dispatch(request, *args, **kwargs)


class TopicView(TemplateView):
    template_name = 'forum/view_topic.html'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_context_data(self, **kwargs):
        context = super(TopicView, self).get_context_data(**kwargs)
        context['topic'] = self.get_object()
        # user_profile = get_object_or_404(UserProfile, user=self.request.user)
        # context['user_profile'] = user_profile
        suggested_topics = Topic.objects.filter(
            category=self.get_object().category).exclude(id=self.get_object().id)
        job_url = 'http://' + self.request.META['HTTP_HOST'] + reverse(
            'django_simple_forum:view_topic', kwargs={'slug': self.get_object().slug})
        try:
            minified_url = google_mini(job_url, settings.MINIFIED_URL)
        except:
            minified_url = job_url

        context['minified_url'] = minified_url
        context['suggested_topics'] = suggested_topics
        return context


def comment_mentioned_users_list(data):
    mentioned_users = data.split(',')
    mentioned_users_list = [user.strip('@') for user in mentioned_users]
    result = User.objects.filter(username__in=mentioned_users_list)
    return result


class CommentAdd(UserMixin, CreateView):
    model = Topic
    form_class = CommentForm
    template_name = 'forum/view_topic.html'

    def form_valid(self, form):
        comment = form.save()
        if self.request.POST['parent']:
            comment.parent_id = self.request.POST['parent']
            comment.save()
        if self.request.POST.get('mentioned_user', False):
            data = self.request.POST.get('mentioned_user')
            comment.mentioned = comment_mentioned_users_list(data)
            comment.save()

        for user in comment.topic.get_topic_users():
            mto = [user.user.email]
            c = Context({'comment': comment, "user": user.user, 'topic_url': settings.HOST_URL+reverse('django_simple_forum:view_topic', kwargs={'slug': comment.topic.slug}), "HOST_URL": settings.HOST_URL})
            t = loader.get_template('emails/comment_add.html')
            subject = "New Comment For The Topic " + (comment.topic.title)
            rendered = t.render(c)
            mfrom = settings.DEFAULT_FROM_EMAIL
            Memail(mto, mfrom, subject, rendered, 'emails/comment_add.html', c)

        for user in comment.mentioned.all():
            mto = [user.user.email]
            c = Context({'comment': comment, "user": user.user, 'topic_url': settings.HOST_URL+reverse('django_simple_forum:view_topic', kwargs={'slug': comment.topic.slug}), "HOST_URL": settings.HOST_URL})
            t = loader.get_template('emails/comment_mentioned.html')
            subject = "New Comment For The Topic " + (comment.topic.title)
            rendered = t.render(c)
            mfrom = settings.DEFAULT_FROM_EMAIL
            Memail(mto, mfrom, subject, rendered, 'emails/comment_mentioned.html', c)

        timeline_activity(user=self.request.user, content_object=comment,
                          namespace='commented for the', event_type="comment-create")

        data = {'error': False, 'response': 'Successfully Created Topic'}
        return JsonResponse(data)

    def get_form(self, form_class):
        return form_class(self.request.POST, user=self.request.user)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:signup'))

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(CommentAdd, self).get_context_data(**kwargs)
        form = CommentForm(self.request.GET)
        context['form'] = form
        return context


class CommentEdit(UserMixin, UpdateView):
    model = Comment
    template_name = "dashboard/edit_user.html"
    form_class = CommentForm
    slug_field = 'slug'

    def get_object(self):
        return get_object_or_404(Comment, id=self.kwargs['comment_id'])

    def form_valid(self, form):
        comment = self.get_object()
        if self.request.user == comment.commented_by:
            self.get_object().mentioned.all().delete()
            comment = form.save()
            if self.request.POST['parent']:
                comment.parent_id = self.request.POST['parent']
                comment.save()
            if self.request.POST.get('mentioned_user', False):
                data = self.request.POST.get('mentioned_user')
                comment.mentioned = comment_mentioned_users_list(data)
                comment.save()
            timeline_activity(user=self.request.user, content_object=comment,
                              namespace='commented for the', event_type="comment-create")
            data = {'error': False, 'response': 'Successfully Edited User'}
        else:
            data = {'error': True, 'response': 'Only Commented User Can edit this comment'}
        return JsonResponse(data)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:users'))

    def get_form(self, form_class):
        return form_class(self.request.POST, instance=self.get_object())

    def form_invalid(self, form):
        print (form.errors)
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(DashboardUserEdit, self).get_context_data(**kwargs)
        form = CommentForm(self.request.GET)
        context['form'] = form
        return context


class CommentDelete(UserMixin, DeleteView):
    model = Comment
    slug_field = 'comment_id'
    template_name = "dashboard/categories.html"

    def get_object(self):
        return get_object_or_404(Comment, id=self.kwargs['comment_id'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:categories'))

    def post(self, request, *args, **kwargs):
        comment = self.get_object()
        if self.request.user == comment.commented_by:
            comment.delete()
            return JsonResponse({'error': False, 'response': 'Successfully Deleted Your Comment'})
        else:
            return JsonResponse({'error': False, 'response': 'Only commented user can delete this comment'})


class TopicLike(UserMixin, View):
    model = Topic
    slug_field = 'slug'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:categories'))

    def post(self, request, *args, **kwargs):
        topic = self.get_object()
        user_topics = UserTopics.objects.filter(user=request.user, topic=topic)
        if user_topics:
            user_topic = user_topics[0]
        else:
            user_topic = UserTopics.objects.create(
                user=request.user, topic=topic)
        if user_topic.is_like:
            user_topic.is_like = False
            topic.no_of_likes = topic.no_of_likes - 1
            timeline_activity(user=self.request.user, content_object=topic,
                              namespace='unlike the', event_type="unlike-topic")
        else:
            user_topic.is_like = True
            topic.no_of_likes = topic.no_of_likes + 1
            timeline_activity(
                user=self.request.user, content_object=topic, namespace='like the', event_type="like-topic")
        user_topic.save()
        topic.save()

        return JsonResponse({'error': False, 'response': 'Successfully Deleted Category', 'is_like': user_topic.is_like, 'no_of_likes': topic.no_of_likes, 'no_of_users': topic.get_topic_users().count()})


class ForumCategoryList(ListView):
    queryset = ForumCategory.objects.filter(
        is_active=True, is_votable=True).order_by('-created_on')
    template_name = 'forum/categories.html'
    context_object_name = "categories"
    paginate_by = '10'


class ForumTagsList(ListView):
    queryset = Tags.objects.filter()
    template_name = 'forum/tags.html'
    context_object_name = "tags"
    paginate_by = '10'

    def post(self, request, *args, **kwargs):
        tags = self.queryset
        if str(request.POST.get('alphabet_value')) != 'all':
            tags = tags.filter(
                title__istartswith=request.POST.get('alphabet_value'))
        return render(request, self.template_name, {'tags': tags})


class ForumBadgeList(ListView):
    queryset = Tags.objects.filter()
    template_name = 'forum/badges.html'
    context_object_name = "badges_list"
    paginate_by = '10'

    def post(self, request, *args, **kwargs):
        tags = self.queryset
        if str(request.POST.get('alphabet_value')) != 'all':
            tags = tags.filter(
                title__istartswith=request.POST.get('alphabet_value'))
        return render(request, self.template_name, {'tags': tags})


class ForumCategoryView(ListView):
    model = Topic
    template_name = 'forum/topic_list.html'

    def get_object(self):
        return get_object_or_404(ForumCategory, slug=self.kwargs['slug'])

    def get_context_data(self, **kwargs):
        context = super(ForumCategoryView, self).get_context_data(**kwargs)
        topics = self.get_object().get_topics()
        context['topic_list'] = topics
        return context

    def post(self, request, *args, **kwargs):
        topic = self.get_object()


class ForumTagsView(ListView):
    model = Topic
    template_name = 'forum/topic_list.html'

    def get_object(self):
        return get_object_or_404(Tags, slug=self.kwargs['slug'])

    def get_context_data(self, **kwargs):
        context = super(ForumTagsView, self).get_context_data(**kwargs)
        topics = self.get_object().get_topics()
        context['topic_list'] = topics
        return context

    def post(self, request, *args, **kwargs):
        topic = self.get_object()


class DashboardTopicDelete(AdminMixin, DeleteView):
    model = Topic
    slug_field = 'slug'
    template_name = "dashboard/topic.html"

    def get_object(self):
        return Topic.objects.filter(slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:badges'))

    def post(self, request, *args, **kwargs):
        topic = self.get_object()
        topic.delete()
        return JsonResponse({'error': False, 'response': 'Successfully Deleted Topic'})


class TopicDetail(AdminMixin, TemplateView):
    template_name = 'dashboard/view_topic.html'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_context_data(self, **kwargs):
        context = super(TopicDetail, self).get_context_data(**kwargs)
        context['topic'] = self.get_object()
        return context


class TopicStatus(AdminMixin, View):
    model = Topic
    slug_field = 'slug'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:topics'))

    def post(self, request, *args, **kwargs):
        topic = self.get_object()
        if topic.status == 'Draft':
            topic.status = 'Published'
        elif topic.status == 'Published':
            topic.status = 'Draft'
        else:
            topic.status = 'Disabled'
        topic.save()
        return JsonResponse({'error': False, 'response': 'Successfully Updated Topic Status'})


class DashboardUserDelete(AdminMixin, DeleteView):
    model = User
    template_name = "dashboard/topic.html"
    slug_name = "user_id"

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:users'))

    def get_object(self):
        return get_object_or_404(User, id=self.kwargs['user_id'])

    def post(self, request, *args, **kwargs):
        user = self.get_object()
        user.delete()
        return JsonResponse({'error': False, 'response': 'Successfully Deleted User'})


class UserStatus(AdminMixin, View):
    model = User
    slug_name = "user_id"

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:users'))

    def get_object(self):
        return get_object_or_404(User, id=self.kwargs['user_id'])

    def post(self, request, *args, **kwargs):
        user = self.get_object()
        if user.is_active:
            user.is_active = False
        else:
            user.is_active = True
        user.save()
        return JsonResponse({'error': False, 'response': 'Successfully Updated User Status'})


class UserDetail(AdminMixin, TemplateView):
    template_name = 'dashboard/view_user.html'

    def get_object(self):
        return get_object_or_404(User, id=self.kwargs['user_id'])

    def get_context_data(self, **kwargs):
        context = super(UserDetail, self).get_context_data(**kwargs)
        context['user'] = self.get_object()
        context['user_profile'] = get_object_or_404(
            UserProfile, user=self.get_object())
        user_topics = UserTopics.objects.filter(user=self.get_object())
        context['user_topics'] = user_topics
        context['user_liked_topics'] = user_topics.filter(is_like=True)
        context['user_followed_topics'] = user_topics.filter(is_followed=True)
        context['user_created_topics'] = Topic.objects.filter(created_by=self.get_object())
        return context


class TopicFollow(UserMixin, View):
    model = Topic
    slug_field = 'slug'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:topic_list'))

    def post(self, request, *args, **kwargs):
        topic = self.get_object()
        user_topics = UserTopics.objects.filter(user=request.user, topic=topic)
        if user_topics:
            user_topic = user_topics[0]
        else:
            user_topic = UserTopics.objects.create(
                user=request.user, topic=topic)
        if user_topic.is_followed:
            user_topic.is_followed = False
            user_topic.followed_on = datetime.now()
            timeline_activity(user=self.request.user, content_object=topic,
                              namespace='unfollow the', event_type="unfollow-topic")
        else:
            user_topic.is_followed = True
            user_topic.followed_on = datetime.now()
            timeline_activity(user=self.request.user, content_object=topic,
                              namespace='follow the', event_type="follow-topic")
        user_topic.save()
        return JsonResponse({'error': False, 'response': 'Successfully Followed the topic', 'is_followed': user_topic.is_followed})


class TopicVotes(UserMixin, View):
    model = Topic
    slug_field = 'slug'

    def get_object(self):
        return get_object_or_404(Topic, slug=self.kwargs['slug'])

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:topic_list'))

    def post(self, request, *args, **kwargs):
        topic = self.get_object()
        user_profile = get_object_or_404(UserProfile, user=request.user)
        user_topics = UserTopics.objects.filter(user=request.user, topic=topic)

        if user_topics:
            user_topic = user_topics[0]
        else:
            user_topic = UserTopics.objects.create(
                user=request.user, topic=topic)

        if user_profile.used_votes < 15:
            if self.request.POST.get('vote') == 'up_vote':
                user_topic.no_of_votes = int(user_topic.no_of_votes) + 1
                topic.no_of_votes = int(topic.no_of_votes) + 1
                user_profile.used_votes = int(user_profile.used_votes) + 1
                if user_topic.no_of_down_votes > 0:
                    user_profile.used_votes = user_profile.used_votes - user_topic.no_of_down_votes
                    topic.no_of_down_votes = topic.no_of_down_votes - user_topic.no_of_down_votes
                    user_topic.no_of_down_votes = 0
                    print (user_topic.no_of_down_votes)

                timeline_activity(
                    user=self.request.user, content_object=topic, namespace='Vote the', event_type="vote-topic")
            elif self.request.POST.get('vote') == 'down_vote':
                user_topic.no_of_down_votes = user_topic.no_of_down_votes + 1
                topic.no_of_down_votes = topic.no_of_down_votes + 1
                user_profile.used_votes = user_profile.used_votes + 1
                if user_topic.no_of_votes > 0:
                    user_profile.used_votes = user_profile.used_votes - user_topic.no_of_votes
                    topic.no_of_votes = topic.no_of_votes - user_topic.no_of_votes
                    user_topic.no_of_votes = 0

                timeline_activity(user=self.request.user, content_object=topic,
                                  namespace='Un vote the', event_type="unvote-topic")

            topic.save()
            user_topic.save()
            user_profile.save()
            return JsonResponse({'error': False,
                                 'response': 'Successfully Followed the topic',
                                 'no_of_votes': topic.no_of_votes,
                                 'no_of_down_votes': topic.no_of_down_votes})
        return JsonResponse({'error': True, 'response': 'You have already used your maximum no of your votes'})


class ChangePassword(AdminMixin, FormView):
    template_name = 'dashboard/change_password.html'
    form_class = ChangePasswordForm

    def form_valid(self, form):
        user = self.request.user
        if not check_password(self.request.POST['oldpassword'], user.password):
            return HttpResponse(json.dumps({'error': True, 'response': {'oldpassword': 'Invalid old password'}}))
        if self.request.POST['newpassword'] != self.request.POST['retypepassword']:
            return HttpResponse(json.dumps({'error': True, 'response': {'newpassword': 'New password and Confirm Passwords did not match'}}))
        user.set_password(self.request.POST['newpassword'])
        user.save()
        return HttpResponse(json.dumps({'error': False, 'message': 'Password changed successfully'}))

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class UserProfileView(UserMixin, TemplateView):
    template_name = 'forum/profile.html'

    def get_context_data(self, **kwargs):
        context = super(UserProfileView, self).get_context_data(**kwargs)
        user_profile = get_object_or_404(UserProfile, user=self.request.user)
        context['user_profile'] = user_profile
        return context


class ProfileView(TemplateView):
    template_name = 'forum/profile.html'
    slug_field = 'user_name'

    def get_context_data(self, **kwargs):
        context = super(ProfileView, self).get_context_data(**kwargs)
        user_profile = get_object_or_404(UserProfile, user__username=self.kwargs['user_name'])
        context['user_profile'] = user_profile
        return context


class UserProfilePicView(UserMixin, View):
    model = UserProfile

    def get_object(self):
        return get_object_or_404(UserProfile, user_id=self.request.user.id)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:user_profile'))

    def post(self, request, *args, **kwargs):
        user_profile = self.get_object()
        if 'profile_pic' in request.FILES:
            user_profile.profile_pic = request.FILES['profile_pic']
            user_profile.save()
            return JsonResponse({'error': False, 'response': 'Successfully uploaded'})
        else:
            return JsonResponse({'error': True, 'response': 'Please Upload Your Profile pic'})


class UserSettingsView(UserMixin, View):
    model = UserProfile

    def get_object(self):
        return get_object_or_404(UserProfile, user_id=self.request.user.id)

    def get_success_url(self):
        return redirect(reverse('django_simple_forum:user_profile'))

    def post(self, request, *args, **kwargs):
        user_profile = self.get_object()
        if not user_profile.send_mailnotifications:
            user_profile.send_mailnotifications = True
        else:
            user_profile.send_mailnotifications = False
        user_profile.save()
        return JsonResponse({'error': False, 'response': 'You have successfully uploaded the settings', "send_mailnotifications": user_profile.send_mailnotifications})


class UserDetailView(TemplateView):
    template_name = 'forum/profile.html'
    slug_field = 'user_name'

    def get_object(self):
        return get_object_or_404(UserProfile, user__username=self.kwargs['user_name'])

    def get_context_data(self, **kwargs):
        context = super(UserDetailView, self).get_context_data(**kwargs)
        context['user_profile'] = self.get_object()
        return context


def facebook_login(request):
    if 'code' in request.GET:
        accesstoken = get_access_token_from_code(request.GET['code'], request.scheme + '://' + request.META['HTTP_HOST'] + reverse('django_simple_forum:facebook_login'), settings.FB_APP_ID, settings.FB_SECRET)
        if 'error' in accesstoken.keys():
            messages.error(request, "Sorry, Your session has been expired")
            return render(request, '404.html')
        graph = GraphAPI(accesstoken['access_token'])
        accesstoken = graph.extend_access_token(settings.FB_APP_ID, settings.FB_SECRET)['accesstoken']
        profile = graph.get_object("me")
        hometown = profile['hometown']['name'] if 'hometown' in profile.keys() else ''
        location = profile['location']['name'] if 'location' in profile.keys() else ''
        bday = datetime.strptime(profile['birthday'], '%m/%d/%Y').strftime('%Y-%m-%d') if 'birthday' in profile.keys() else '1970-09-09'
        profile_pic = "https://graph.facebook.com/" + \
            profile['id'] + "/picture?type=large"

        if 'email' in profile.keys():
            if User.objects.filter(username=profile['email'], email=profile['email']):
                user = User.objects.filter(username=profile['email'], email=profile['email']).first()
            else:
                user, created = User.objects.get_or_create(
                    username=profile['email'],
                    email=profile['email'],
                    first_name=profile['first_name'],
                    last_name=profile['last_name'],
                    last_login=datetime.now()
                )
            user_profile = UserProfile.objects.filter(user=user).last()
            if not user_profile:
                user_profile = UserProfile.objects.create(user=user, user_roles="Publisher")
            name = urlparse(profile_pic).path.split('/')[-1]
            # content = urllib.urlretrieve(profile_pic)
            content = urllib.request.urlretrieve(profile_pic)
            user_profile.profile_pic.save(name, File(open(content[0], 'rb')), save=True)
            user_profile.save()

            fb = Facebook.objects.filter(user=user).last()
            if not fb:
                Facebook.objects.create(
                    user=user,
                    facebook_url=profile['link'],
                    facebook_id=profile['id'],
                    first_name=profile['first_name'],
                    last_name=profile['last_name'],
                    verified=profile['verified'],
                    name=profile['name'],
                    language=profile['locale'],
                    hometown=hometown,
                    email=profile['email'],
                    gender=profile['gender'],
                    dob=bday,
                    location=location,
                    timezone=profile['timezone'],
                    accesstoken=accesstoken
                )
            else:
                fb.user = user
                fb.facebook_url = profile['link']
                fb.facebook_id = profile['id']
                fb.first_name = profile['first_name']
                fb.last_name = profile['last_name']
                fb.verified = profile['verified']
                fb.name = profile['name']
                fb.language = profile['locale']
                fb.hometown = hometown
                fb.email = profile['email']
                fb.gender = profile['gender']
                fb.dob = bday
                fb.location = location
                fb.timezone = profile['timezone']
                fb.accesstoken = accesstoken
                fb.save()
            if not request.user.is_authenticated():
                if not hasattr(user, 'backend'):
                    for backend in settings.AUTHENTICATION_BACKENDS:
                        if user == load_backend(backend).get_user(user.pk):
                            user.backend = backend
                            break
                if hasattr(user, 'backend'):
                    login(request, user)
            return HttpResponseRedirect('/forum/')
        else:
            message.error(request, "Sorry, We didnt find your email id through facebook")
            return render(request, '404.html')
    elif 'error' in request.GET:
        print(request.GET)
    else:
        rty = "https://graph.facebook.com/oauth/authorize?client_id=" + settings.FB_APP_ID + "&redirect_uri=" + request.scheme + '://' + request.META['HTTP_HOST'] + reverse('django_simple_forum:facebook_login') + "&scope=manage_pages,read_stream, user_about_me, user_birthday, user_location, user_work_history, user_hometown, user_website, email, user_likes, user_groups"
        return HttpResponseRedirect(rty)


def google_login(request):
    if 'code' in request.GET:
        params = {
            'grant_type': 'authorization_code',
            'code': request.GET.get('code'),
            'redirect_uri': request.scheme + "://" + request.META['HTTP_HOST'] + reverse('django_simple_forum:google_login'),
            'client_id': settings.GP_CLIENT_ID,
            'client_secret': settings.GP_CLIENT_SECRET
        }
        info = requests.post("https://accounts.google.com/o/oauth2/token", data=params)
        info = info.json()
        url = 'https://www.googleapis.com/oauth2/v1/userinfo'
        params = {'access_token': info['access_token']}
        kw = dict(params=params, headers={}, timeout=60)
        response = requests.request('GET', url, **kw)
        user_document = response.json()
        link = "https://plus.google.com/" + user_document['id']
        picture = user_document['picture'] if 'picture' in user_document.keys() else ""
        dob = user_document['birthday'] if 'birthday' in user_document.keys() else ""
        gender = user_document['gender'] if 'gender' in user_document.keys() else ""
        link = user_document['link'] if 'link' in user_document.keys() else link

        if request.user.is_authenticated():
            user = request.user
        else:
            user = User.objects.filter(email=user_document['email']).first()
        if user:
            user.first_name = user_document['name']
            user.last_name = user_document['family_name']
            user.save()
        else:
            user = User.objects.create(
                username=user_document['email'],
                email=user_document['email'],
                first_name=user_document['name'],
                last_name=user_document['family_name'],
            )
        user.save()

        user_profile = UserProfile.objects.filter(user=user).last()
        if not user_profile:
            user_profile = UserProfile.objects.create(user=user, user_roles="Publisher")
        google, created = Google.objects.get_or_create(user=user)
        google.user = user
        google.google_url = link
        google.verified_email = user_document['verified_email']
        google.google_id = user_document['id']
        google.family_name = user_document['family_name']
        google.name = user_document['name']
        google.given_name = user_document['given_name']
        google.dob = dob
        google.email = user_document['email']
        google.gender = gender
        google.picture = picture
        google.save()

        if not request.user.is_authenticated():
            if not hasattr(user, 'backend'):
                for backend in settings.AUTHENTICATION_BACKENDS:
                    if user == load_backend(backend).get_user(user.pk):
                        user.backend = backend
                        break
            if hasattr(user, 'backend'):
                login(request, user)
        return HttpResponseRedirect('/forum/')
    else:
        rty = "https://accounts.google.com/o/oauth2/auth?client_id=" + settings.GP_CLIENT_ID\
              + "&response_type=code"
        rty += "&scope=https://www.googleapis.com/auth/userinfo.profile \
               https://www.googleapis.com/auth/userinfo.email&redirect_uri=" + request.scheme\
               + "://" + request.META['HTTP_HOST'] + reverse('django_simple_forum:google_login')\
               + "&state=1235dfghjkf123"
        return HttpResponseRedirect(rty)


def get_mentioned_user(request, topic_id):
    topic = get_object_or_404(Topic, id=topic_id)
    if request.method == 'GET':
        topic_users = topic.get_topic_users()
        list_data = []
        for user in topic_users:
            data = {}
            data['username'] = user.user.email.split('@')[0]
            # data['avatar'] = user.profile_pic.url if user.profile_pic else 'https://www.2025ad.com/typo3conf/ext/providerconti/Resources/Public/src/Images/profil-pic_dummy.png'
            data['fullname'] = user.user.email
            list_data.append(data)
    return HttpResponse(
        json.dumps({'data': list_data}), content_type="application/json")


def comment_mentioned_users_list(data):
    mentioned_users = data.split(',')
    mentioned_users_list = [user.strip('@') for user in mentioned_users]
    result = User.objects.filter(username__in=mentioned_users_list)
    return result


class UserChangePassword(UserMixin, FormView):
    template_name = 'forum/topic_list.html'
    form_class = UserChangePasswordForm

    def form_valid(self, form):
        user = self.request.user
        if self.request.POST['newpassword'] != self.request.POST['retypepassword']:
            return HttpResponse(json.dumps({'error': True, 'response': {'newpassword': 'New password and Confirm Passwords did not match'}}))
        user.set_password(self.request.POST['newpassword'])
        user.save()
        return HttpResponse(json.dumps({'error': False, 'message': 'Password changed successfully'}))

    def form_invalid(self, form):
        print (form.errors)
        return JsonResponse({'error': True, 'response': form.errors})


class ForgotPasswordView(FormView):
    template_name = 'form/topic_list.html'
    form_class = ForgotPasswordForm

    def form_valid(self, form):
        user = User.objects.filter(email=self.request.POST.get('email'))
        if user:
            user = user[0]
            subject = "Password Reset"
            password = get_random_string(6)
            message = '<p>Your Password for the forum account is <strong>'+password+'</strong></p><br/><p>Use this credentials to login into <a href="' + settings.HOST_URL + '/forum/">forum</a></p>'
            to = user.email
            from_email = settings.DEFAULT_FROM_EMAIL
            Memail(to, from_email, subject, message, '', '')
            user.set_password(password)
            user.save()
            data = {"error": False, "response": "An Email is sent to the entered email id"}
            return HttpResponse(json.dumps(data))
        else:
            data = {"error": True, "message": "User With this email id doesn't exists!!!"}
            return HttpResponse(json.dumps(data))

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-28 12:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_forum', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='forumcategory',
            name='slug',
            field=models.SlugField(default='test', max_length=1000),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='topic',
            name='slug',
            field=models.SlugField(default='test', max_length=1000),
            preserve_default=False,
        ),
    ]



from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
import hashlib
from datetime import datetime

STATUS = (
    ('Draft', 'Draft'),
    ('Published', 'Published'),
    ('Disabled', 'Disabled'),
)

USER_ROLES = (
    ('Admin', 'Admin'),
    ('Publisher', 'Publisher'),
)

User = settings.AUTH_USER_MODEL


# tags created for topic
class Tags(models.Model):
    title = models.CharField(max_length=50, unique=True)
    slug = models.CharField(max_length=50, unique=True)

    def get_topics(self):
        topics = Topic.objects.filter(tags__in=[self], status='Published')
        return topics


# tags created for topic
class Badge(models.Model):
    title = models.CharField(max_length=50, unique=True)
    slug = models.CharField(max_length=50, unique=True)

    def get_users(self):
        user_profile = UserProfile.objects.filter(badges__in=[self])
        return user_profile


def img_url(self, filename):
    hash_ = hashlib.md5()
    hash_.update(
        str(filename).encode('utf-8') + str(datetime.now()).encode('utf-8'))
    file_hash = hash_.hexdigest()
    return "%s%s/%s" % (self.file_prepend, file_hash, filename)


# user profile to store no of votes available to user, badges for a topic, user roles,
class UserProfile(models.Model):
    user = models.ForeignKey(User)
    used_votes = models.IntegerField(default='0')
    user_roles = models.CharField(choices=USER_ROLES, max_length=10)
    badges = models.ManyToManyField(Badge)
    file_prepend = "forum_user/profilepics/"
    profile_pic = models.FileField(
        max_length=500, null=True, blank=True, upload_to=img_url)
    send_mailnotifications = models.BooleanField(default=False)

    # need to add social details for a user if we implement socail login

    def get_no_of_up_votes(self):
        user_topics = UserTopics.objects.filter(user=self.user)
        votes = 0
        for topic in user_topics:
            votes += topic.no_of_votes
        return votes

    def get_no_of_down_votes(self):
        user_topics = UserTopics.objects.filter(user=self.user)
        votes = 0
        for topic in user_topics:
            votes += topic.no_of_down_votes
        return votes

    def get_topics(self):
        topics = Topic.objects.filter(created_by=self.user)
        return topics

    def get_followed_topics(self):
        topics = UserTopics.objects.filter(user=self.user, is_followed=True)
        topics = Topic.objects.filter(id__in=topics.values_list('topic', flat=True))
        return topics

    def get_liked_topics(self):
        topics = UserTopics.objects.filter(user=self.user, is_like=True)
        topics = Topic.objects.filter(id__in=topics.values_list('topic', flat=True))
        return topics

    def get_timeline(self):
        timeline = Timeline.objects.filter(user=self.user).order_by('-created_on')
        return timeline

    def get_user_topic_tags(self):
        tags = Tags.objects.filter(id__in=self.get_topics().values_list('tags', flat=True))
        return tags

    def get_user_topic_categories(self):
        categories = ForumCategory.objects.filter(id__in=self.get_topics().values_list('category', flat=True))
        return categories
        # return []

    def get_user_suggested_topics(self):
        categories = ForumCategory.objects.filter(id__in=self.get_topics().values_list('category', flat=True))
        topics = Topic.objects.filter(category__id__in=categories.values_list('id', flat=True))
        return topics
        # return []


class ForumCategory(models.Model):
    created_by = models.ForeignKey(User)
    title = models.CharField(max_length=1000)
    is_active = models.BooleanField(default=False)
    color = models.CharField(max_length=20, default="#999999")
    is_votable = models.BooleanField(default=False)
    created_on = models.DateTimeField(auto_now_add=True)
    slug = models.SlugField(max_length=1000)
    description = models.TextField()
    parent = models.ForeignKey('self', blank=True, null=True)

    def get_topics(self):
        topics = Topic.objects.filter(category=self, status='Published')
        return topics


class Topic(models.Model):
    title = models.CharField(max_length=2000)
    description = models.TextField()
    created_by = models.ForeignKey(User)
    status = models.CharField(choices=STATUS, max_length=10)
    category = models.ForeignKey(ForumCategory)
    created_on = models.DateTimeField(auto_now=True)
    updated_on = models.DateTimeField(auto_now=True)
    no_of_views = models.IntegerField(default='0')
    slug = models.SlugField(max_length=1000)
    tags = models.ManyToManyField(Tags)
    no_of_likes = models.IntegerField(default='0')
    no_of_votes = models.IntegerField(default='0')
    no_of_down_votes = models.IntegerField(default='0')

    def get_comments(self):
        comments = Comment.objects.filter(topic=self, parent=None)
        return comments

    def get_all_comments(self):
        comments = Comment.objects.filter(topic=self)
        return comments

    def get_last_comment(self):
        comments = Comment.objects.filter(topic=self).order_by('-updated_on').first()
        return comments

    def get_topic_users(self):
        comment_user_ids = Comment.objects.filter(topic=self).values_list('commented_by', flat=True)
        liked_users_ids = UserTopics.objects.filter(topic=self, is_like=True).values_list('user', flat=True)
        followed_users = UserTopics.objects.filter(topic=self, is_followed=True).values_list('user', flat=True)
        all_users = list(comment_user_ids) + list(liked_users_ids) + list(followed_users) + [self.created_by.id]
        users = UserProfile.objects.filter(user_id__in=set(all_users))
        return users

    def get_total_of_votes(self):
        no_of_votes = self.no_of_votes + self.no_of_down_votes
        return no_of_votes


# user followed topics
class UserTopics(models.Model):
    user = models.ForeignKey(User)
    topic = models.ForeignKey(Topic)
    is_followed = models.BooleanField(default=False)
    followed_on = models.DateField(null=True, blank=True)
    no_of_votes = models.IntegerField(default='0')
    no_of_down_votes = models.IntegerField(default='0')
    is_like = models.BooleanField(default=False)


class Comment(models.Model):
    comment = models.TextField(null=True, blank=True)
    commented_by = models.ForeignKey(User, related_name="commented_by")
    topic = models.ForeignKey(Topic, related_name="topic_comments")
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now_add=True)
    parent = models.ForeignKey("self", blank=True, null=True, related_name="comment_parent")
    mentioned = models.ManyToManyField(User, related_name="mentioned_users")

    def get_comments(self):
        comments = self.comment_parent.all()
        return comments


# user activity
class Timeline(models.Model):
    content_type = models.ForeignKey(ContentType, related_name="content_type_timelines")
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")
    namespace = models.CharField(max_length=250, default="default", db_index=True)
    event_type = models.CharField(max_length=250, db_index=True)
    user = models.ForeignKey(User, null=True)
    data = models.TextField(null=True, blank=True)
    created_on = models.DateTimeField(auto_now_add=True)
    is_read = models.BooleanField(default=False)

    class Meta:
        index_together = [("content_type", "object_id", "namespace"), ]
        ordering = ['-created_on']


class Attachment(models.Model):
    file_prepend = "forum_topic/attachments/"
    uploaded_by = models.ForeignKey(User, related_name='attachments_user')
    created_on = models.DateTimeField(auto_now_add=True)
    attached_file = models.FileField(
        max_length=500, null=True, blank=True, upload_to=img_url)
    comment = models.ForeignKey(Comment)


class Facebook(models.Model):
    user = models.ForeignKey(User, related_name='user_facebook')
    facebook_id = models.CharField(max_length=100)
    facebook_url = models.CharField(max_length=200, default='')
    first_name = models.CharField(max_length=200, default='')
    last_name = models.CharField(max_length=200, default='')
    verified = models.CharField(max_length=200, default='')
    name = models.CharField(max_length=200, default='')
    language = models.CharField(max_length=200, default='')
    hometown = models.CharField(max_length=200, default='')
    email = models.CharField(max_length=200, default='', db_index=True)
    gender = models.CharField(max_length=200, default='')
    dob = models.DateField(null=True, blank=True)
    location = models.CharField(max_length=200, default='')
    timezone = models.CharField(max_length=200, default='')
    accesstoken = models.CharField(max_length=2000, default='')


class Google(models.Model):
    user = models.ForeignKey(User, related_name='user_google')
    google_id = models.CharField(max_length=200, default='')
    google_url = models.CharField(max_length=1000, default='')
    verified_email = models.CharField(max_length=200, default='')
    family_name = models.CharField(max_length=200, default='')
    name = models.CharField(max_length=200, default='')
    picture = models.CharField(max_length=200, default='')
    gender = models.CharField(max_length=10, default='')
    dob = models.CharField(max_length=50, default='')
    given_name = models.CharField(max_length=200, default='')
    email = models.CharField(max_length=200, default='', db_index=True)


class GitHub(models.Model):
    user = models.ForeignKey(User, related_name='user_github')
    git_url = models.URLField()
    git_id = models.CharField(max_length=50)
    disk_usage = models.CharField(max_length=200)
    private_gists = models.IntegerField(default=0)
    public_gists = models.IntegerField(default=0)
    public_repos = models.IntegerField(default=0)
    hireable = models.BooleanField(default=False)
    total_private_repos = models.IntegerField(default=0)
    owned_private_repos = models.IntegerField(default=0)
    following = models.IntegerField(default=0)
    followers = models.IntegerField(default=0)
    company = models.CharField(max_length=200)
    name = models.CharField(max_length=200)
    user_from = models.DateTimeField()


class Twitter(models.Model):
    user = models.ForeignKey(User, related_name='user_twitter')
    twitter_id = models.CharField(max_length=100, default='')
    screen_name = models.CharField(max_length=100, default='')
    oauth_token = models.CharField(max_length=200, default='')
    oauth_secret = models.CharField(max_length=200, default='')



from django.conf import settings
import boto.ses
import sendgrid
import requests
from django.core.mail import EmailMultiAlternatives
from django_ses_gateway.sending_mail import sending_mail


def Memail(mto, mfrom, msubject, mbody, email_template_name, context):
    if settings.MAIL_SENDER == 'AMAZON':
        sending_mail(msubject, email_template_name, context, mfrom, mto)
    elif settings.MAIL_SENDER == 'MAILGUN':
        requests.post(
            settings.MGUN_API_URL,
            auth=('api', settings.MGUN_API_KEY),
            data={
                'from': mfrom,
                'to': mto,
                'subject': msubject,
                'html': mbody
            })
    elif settings.MAIL_SENDER == 'SENDGRID':
        sg = sendgrid.SendGridClient(settings.SG_USER, settings.SG_PWD)
        sending_msg = sendgrid.Mail()
        sending_msg.set_subject(msubject)
        sending_msg.set_html(mbody)
        sending_msg.set_text(msubject)
        sending_msg.set_from(mfrom)
        sending_msg.add_to(mto)
        reposne = sg.send(sending_msg)
        print (reposne)
    else:
        text_content = msubject
        msg = EmailMultiAlternatives(msubject, mbody, mfrom, mto)
        msg.attach_alternative(mbody, "text/html")
        msg.send()
